@@ -10,12 +10,46 @@ from concurrent.futures import Future
 from awscrt import NativeResource
 from awscrt.http import HttpRequest
 from awscrt.io import ClientBootstrap, TlsConnectionOptions
-from awscrt.auth import AwsCredentialsProvider
+from awscrt.auth import AwsCredentialsProvider, AwsSignatureType, AwsSignedBodyHeaderType, AwsSignedBodyValue, \
+    AwsSigningAlgorithm, AwsSigningConfig
 import awscrt.exceptions
+from dataclasses import dataclass
 import threading
+from typing import Optional
 from enum import IntEnum
 
 
+class CrossProcessLock(NativeResource):
+    """
+    Class representing an exclusive cross-process lock, scoped by `lock_scope_name`
+
+    Recommended usage is to either explicitly call acquire() followed by release() when the lock  is no longer required, or use this in a 'with' statement.
+
+    acquire() will throw a RuntimeError with AWS_MUTEX_CALLER_NOT_OWNER as the error code, if the lock could not be acquired.
+
+    If the lock has not been explicitly released when the process exits, it will be released by the operating system.
+
+    Keyword Args:
+        lock_scope_name (str): Unique string identifying the caller holding the lock.
+    """
+
+    def __init__(self, lock_scope_name):
+        super().__init__()
+        self._binding = _awscrt.s3_cross_process_lock_new(lock_scope_name)
+
+    def acquire(self):
+        _awscrt.s3_cross_process_lock_acquire(self._binding)
+
+    def __enter__(self):
+        self.acquire()
+
+    def release(self):
+        _awscrt.s3_cross_process_lock_release(self._binding)
+
+    def __exit__(self, exc_type, exc_value, exc_tb):
+        self.release()
+
+
 class S3RequestType(IntEnum):
     """The type of the AWS S3 request"""
 
@@ -50,6 +84,60 @@ class S3RequestTlsMode(IntEnum):
     """
 
 
+class S3ChecksumAlgorithm(IntEnum):
+    """
+    Checksum algorithm used to verify object integrity.
+    https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html
+    """
+
+    CRC32C = 1
+    """CRC32C"""
+
+    CRC32 = 2
+    """CRC32"""
+
+    SHA1 = 3
+    """SHA-1"""
+
+    SHA256 = 4
+    """SHA-256"""
+
+
+class S3ChecksumLocation(IntEnum):
+    """Where to put the checksum."""
+
+    HEADER = 1
+    """
+    Add checksum as a request header field.
+    The checksum is calculated before any part of the request is sent to the server.
+    """
+
+    TRAILER = 2
+    """
+    Add checksum as a request trailer field.
+    The checksum is calculated as the body is streamed to the server, then
+    added as a trailer field. This may be more efficient than HEADER, but
+    can only be used with "streaming" requests that support it.
+    """
+
+
+@dataclass
+class S3ChecksumConfig:
+    """Configures how the S3Client calculates and verifies checksums."""
+
+    algorithm: Optional[S3ChecksumAlgorithm] = None
+    """
+    If set, the S3Client will calculate a checksum using this algorithm
+    and add it to the request. If you set this, you must also set `location`.
+    """
+
+    location: Optional[S3ChecksumLocation] = None
+    """Where to put the request checksum."""
+
+    validate_response: bool = False
+    """Whether to retrieve and validate response checksums."""
+
+
 class S3Client(NativeResource):
     """S3 client
 
@@ -68,19 +156,30 @@ class S3Client(NativeResource):
             If this is :attr:`S3RequestTlsMode.DISABLED`:
                 No TLS options will be used, regardless of `tls_connection_options` value.
 
-        credential_provider (Optional[AwsCredentialsProvider]): Credentials providers source the
-            :class:`~awscrt.auth.AwsCredentials` needed to sign an authenticated AWS request.
+        signing_config (Optional[AwsSigningConfig]):
+             Configuration for signing of the client. Use :func:`create_default_s3_signing_config()` to create the default config.
+             If None is provided, the request will not be signed.
+
+        credential_provider (Optional[AwsCredentialsProvider]): Deprecated, prefer `signing_config` instead.
+            Credentials providers source the :class:`~awscrt.auth.AwsCredentials` needed to sign an authenticated AWS request.
             If None is provided, the request will not be signed.
 
         tls_connection_options (Optional[TlsConnectionOptions]): Optional TLS Options to be used
             for each connection, unless `tls_mode` is :attr:`S3RequestTlsMode.DISABLED`
 
         part_size (Optional[int]): Size, in bytes, of parts that files will be downloaded or uploaded in.
-            Note: for :attr:`S3RequestType.PUT_OBJECT` request, S3 requires the part size greater than 5MB.
-            (5*1024*1024 by default)
+            Note: for :attr:`S3RequestType.PUT_OBJECT` request, S3 requires the part size greater than 5 MiB.
+            (8*1024*1024 by default)
 
-        throughput_target_gbps (Optional[float]): Throughput target in Gbps that we are trying to reach.
-            (5 Gbps by default)
+        multipart_upload_threshold (Optional[int]): The size threshold in bytes, for when to use multipart uploads.
+            Uploads over this size will use the multipart upload strategy.
+            Uploads this size or less will use a single request.
+            If not set, `part_size` is used as the threshold.
+
+        throughput_target_gbps (Optional[float]): Throughput target in
+            Gigabits per second (Gbps) that we are trying to reach.
+            You can also use `get_recommended_throughput_target_gbps()` to get recommended value for your system.
+            10.0 Gbps by default (may change in future)
     """
 
     __slots__ = ('shutdown_event', '_region')
@@ -91,12 +190,15 @@ class S3Client(NativeResource):
             bootstrap,
             region,
             tls_mode=None,
+            signing_config=None,
             credential_provider=None,
             tls_connection_options=None,
             part_size=None,
+            multipart_upload_threshold=None,
             throughput_target_gbps=None):
         assert isinstance(bootstrap, ClientBootstrap) or bootstrap is None
         assert isinstance(region, str)
+        assert isinstance(signing_config, AwsSigningConfig) or signing_config is None
         assert isinstance(credential_provider, AwsCredentialsProvider) or credential_provider is None
         assert isinstance(tls_connection_options, TlsConnectionOptions) or tls_connection_options is None
         assert isinstance(part_size, int) or part_size is None
@@ -106,46 +208,58 @@ class S3Client(NativeResource):
             throughput_target_gbps,
             float) or throughput_target_gbps is None
 
+        if credential_provider and signing_config:
+            raise ValueError("'credential_provider' has been deprecated in favor of 'signing_config'.  "
+                             "Both parameters may not be set.")
+
         super().__init__()
 
         shutdown_event = threading.Event()
 
         def on_shutdown():
             shutdown_event.set()
+
         self._region = region
         self.shutdown_event = shutdown_event
 
         if not bootstrap:
             bootstrap = ClientBootstrap.get_or_create_static_default()
-        s3_client_core = _S3ClientCore(bootstrap, credential_provider, tls_connection_options)
+
+        s3_client_core = _S3ClientCore(bootstrap, credential_provider, signing_config, tls_connection_options)
 
         # C layer uses 0 to indicate defaults
         if tls_mode is None:
             tls_mode = 0
         if part_size is None:
             part_size = 0
+        if multipart_upload_threshold is None:
+            multipart_upload_threshold = 0
         if throughput_target_gbps is None:
             throughput_target_gbps = 0
 
         self._binding = _awscrt.s3_client_new(
             bootstrap,
+            signing_config,
             credential_provider,
             tls_connection_options,
             on_shutdown,
             region,
             tls_mode,
             part_size,
+            multipart_upload_threshold,
             throughput_target_gbps,
             s3_client_core)
 
     def make_request(
             self,
             *,
-            request,
             type,
-            credential_provider=None,
+            request,
             recv_filepath=None,
             send_filepath=None,
+            signing_config=None,
+            credential_provider=None,
+            checksum_config=None,
             on_headers=None,
             on_body=None,
             on_done=None,
@@ -155,15 +269,11 @@ class S3Client(NativeResource):
         requests under the hood for acceleration.
 
         Keyword Args:
-            request (HttpRequest): The overall outgoing API request for S3 operation.
-                If the request body is a file, set send_filepath for better performance.
-
             type (S3RequestType): The type of S3 request passed in,
                 :attr:`~S3RequestType.GET_OBJECT`/:attr:`~S3RequestType.PUT_OBJECT` can be accelerated
 
-            credential_provider (Optional[AwsCredentialsProvider]): Credentials providers source the
-                :class:`~awscrt.auth.AwsCredentials` needed to sign an authenticated AWS request, for this request only.
-                If None is provided, the credential provider in the client will be used.
+            request (HttpRequest): The overall outgoing API request for S3 operation.
+                If the request body is a file, set send_filepath for better performance.
 
             recv_filepath (Optional[str]): Optional file path. If set, the
                 response body is written directly to a file and the
@@ -175,6 +285,16 @@ class S3Client(NativeResource):
                 request's `body_stream` is ignored. This should give better
                 performance than reading a file from a stream.
 
+            signing_config (Optional[AwsSigningConfig]):
+                Configuration for signing of the request to override the configuration from client. Use :func:`create_default_s3_signing_config()` to create the default config.
+                If None is provided, the client configuration will be used.
+
+            credential_provider (Optional[AwsCredentialsProvider]):  Deprecated, prefer `signing_config` instead.
+                Credentials providers source the :class:`~awscrt.auth.AwsCredentials` needed to sign an authenticated AWS request, for this request only.
+                If None is provided, the client configuration will be used.
+
+            checksum_config (Optional[S3ChecksumConfig]): Optional checksum settings.
+
             on_headers: Optional callback invoked as the response received, and even the API request
                 has been split into multiple parts, this callback will only be invoked once as
                 it's just making one API request to S3.
@@ -209,10 +329,16 @@ class S3Client(NativeResource):
                         failed because server side sent an unsuccessful response, the headers
                         of the response is provided here. Else None will be returned.
 
-                    *   `error_body` (Optional[Bytes]): If request failed because server
+                    *   `error_body` (Optional[bytes]): If request failed because server
                         side sent an unsuccessful response, the body of the response is
                         provided here. Else None will be returned.
 
+                    *   `status_code` (Optional[int]): HTTP response status code (if available).
+                        If request failed because server side sent an unsuccessful response,
+                        this is its status code. If the operation was successful,
+                        this is the final response's status code. If the operation
+                        failed for another reason, None is returned.
+
                     *   `**kwargs` (dict): Forward-compatibility kwargs.
 
             on_progress: Optional callback invoked when part of the transfer is done to report the progress.
@@ -227,11 +353,13 @@ class S3Client(NativeResource):
         """
         return S3Request(
             client=self,
-            request=request,
             type=type,
-            credential_provider=credential_provider,
+            request=request,
             recv_filepath=recv_filepath,
             send_filepath=send_filepath,
+            signing_config=signing_config,
+            credential_provider=credential_provider,
+            checksum_config=checksum_config,
             on_headers=on_headers,
             on_body=on_body,
             on_done=on_done,
@@ -259,11 +387,13 @@ class S3Request(NativeResource):
             self,
             *,
             client,
-            request,
             type,
-            credential_provider=None,
+            request,
             recv_filepath=None,
             send_filepath=None,
+            signing_config=None,
+            credential_provider=None,
+            checksum_config=None,
             on_headers=None,
             on_body=None,
             on_done=None,
@@ -280,10 +410,21 @@ class S3Request(NativeResource):
         self._finished_future = Future()
         self.shutdown_event = threading.Event()
 
+        checksum_algorithm = 0  # 0 means NONE in C
+        checksum_location = 0  # 0 means NONE in C
+        validate_response_checksum = False
+        if checksum_config is not None:
+            if checksum_config.algorithm is not None:
+                checksum_algorithm = checksum_config.algorithm.value
+            if checksum_config.location is not None:
+                checksum_location = checksum_config.location.value
+            validate_response_checksum = checksum_config.validate_response
+
         s3_request_core = _S3RequestCore(
             request,
             self._finished_future,
             self.shutdown_event,
+            signing_config,
             credential_provider,
             on_headers,
             on_body,
@@ -295,10 +436,14 @@ class S3Request(NativeResource):
             client,
             request,
             type,
+            signing_config,
             credential_provider,
             recv_filepath,
             send_filepath,
             region,
+            checksum_algorithm,
+            checksum_location,
+            validate_response_checksum,
             s3_request_core)
 
     @property
@@ -316,9 +461,11 @@ class _S3ClientCore:
 
     def __init__(self, bootstrap,
                  credential_provider=None,
+                 signing_config=None,
                  tls_connection_options=None):
         self._bootstrap = bootstrap
         self._credential_provider = credential_provider
+        self._signing_config = signing_config
         self._tls_connection_options = tls_connection_options
 
 
@@ -332,6 +479,7 @@ class _S3RequestCore:
             request,
             finish_future,
             shutdown_event,
+            signing_config=None,
             credential_provider=None,
             on_headers=None,
             on_body=None,
@@ -339,6 +487,7 @@ class _S3RequestCore:
             on_progress=None):
 
         self._request = request
+        self._signing_config = signing_config
         self._credential_provider = credential_provider
 
         self._on_headers_cb = on_headers
@@ -360,20 +509,97 @@ class _S3RequestCore:
     def _on_shutdown(self):
         self._shutdown_event.set()
 
-    def _on_finish(self, error_code, error_headers, error_body):
+    def _on_finish(self, error_code, status_code, error_headers, error_body):
+        # If C layer gives status_code 0, that means "unknown"
+        if status_code == 0:
+            status_code = None
+
         error = None
         if error_code:
             error = awscrt.exceptions.from_code(error_code)
             if error_body:
                 # TODO The error body is XML, will need to parse it to something prettier.
-                extra_message = ". Body from error request is: " + str(error_body)
-                error.message = error.message + extra_message
+                try:
+                    extra_message = ". Body from error request is: " + str(error_body)
+                    error.message = error.message + extra_message
+                except BaseException:
+                    pass
             self._finished_future.set_exception(error)
         else:
             self._finished_future.set_result(None)
         if self._on_done_cb:
-            self._on_done_cb(error=error, error_headers=error_headers, error_body=error_body)
+            self._on_done_cb(error=error, error_headers=error_headers, error_body=error_body, status_code=status_code)
 
     def _on_progress(self, progress):
         if self._on_progress_cb:
             self._on_progress_cb(progress)
+
+
+def create_default_s3_signing_config(*, region: str, credential_provider: AwsCredentialsProvider, **kwargs):
+    """Create a default `AwsSigningConfig` for S3 service.
+
+        Attributes:
+            region (str): The region to sign against.
+
+            credential_provider (AwsCredentialsProvider): Credentials provider
+                to fetch signing credentials with.
+
+            `**kwargs`: Forward compatibility kwargs.
+
+        Returns:
+            AwsSigningConfig
+    """
+    return AwsSigningConfig(
+        algorithm=AwsSigningAlgorithm.V4,
+        signature_type=AwsSignatureType.HTTP_REQUEST_HEADERS,
+        service="s3",
+        signed_body_header_type=AwsSignedBodyHeaderType.X_AMZ_CONTENT_SHA_256,
+        signed_body_value=AwsSignedBodyValue.UNSIGNED_PAYLOAD,
+        region=region,
+        credentials_provider=credential_provider,
+        use_double_uri_encode=False,
+        should_normalize_uri_path=False,
+    )
+
+
+def get_ec2_instance_type():
+    """
+        First this function will check it's running on EC2 via. attempting to read DMI info to avoid making IMDS calls.
+
+        If the function detects it's on EC2, and it was able to detect the instance type without a call to IMDS
+        it will return it.
+
+        Finally, it will call IMDS and return the instance type from there.
+        Note that in the case of the IMDS call, a new client stack is spun up using 1 background thread. The call is made
+        synchronously with a 1 second timeout: It's not cheap. To make this easier, the underlying result is cached
+        internally and will be freed when this module is unloaded is called.
+
+        Returns:
+           A string indicating the instance type or None if it could not be determined.
+    """
+    return _awscrt.s3_get_ec2_instance_type()
+
+
+def is_optimized_for_system():
+    """
+        Returns:
+            true if the current build of this module has an optimized configuration
+            for the current system.
+    """
+    return _awscrt.s3_is_crt_s3_optimized_for_system()
+
+
+def get_recommended_throughput_target_gbps() -> Optional[float]:
+    """
+    Returns:
+        Recommended throughput, in gigabits per second, based on detected system configuration.
+        If the best throughput configuration is unknown, returns None.
+        Use this as the S3Client's `throughput_target_gbps`.
+    """
+    # Currently the CRT returns 0 if it was unable to make a good guess on configuration. Pre-known configs, have this value set.
+    # Eventually, the CRT will make a full calculation based on NIC and CPU configuration, but until then handle 0.
+    max_value = _awscrt.s3_get_recommended_throughput_target_gbps()
+    if max_value > 0:
+        return max_value
+    else:
+        return None
