@@ -52,6 +52,7 @@ struct aws_event_stream_rpc_client_continuation_token {
     void *user_data;
     struct aws_atomic_var ref_count;
     struct aws_atomic_var is_closed;
+    struct aws_atomic_var is_complete;
 };
 
 static void s_on_message_received(struct aws_event_stream_message *message, int error_code, void *user_data);
@@ -201,17 +202,21 @@ static int s_mark_each_continuation_closed(void *context, struct aws_hash_elemen
 /* Invoke continuation's on_closed() callback.
  * A lock must NOT be hold while calling this */
 static void s_complete_continuation(struct aws_event_stream_rpc_client_continuation_token *token) {
-    AWS_LOGF_DEBUG(
-        AWS_LS_EVENT_STREAM_RPC_CLIENT,
-        "token=%p: token with stream-id %" PRIu32 ", purged from the stream table",
-        (void *)token,
-        token->stream_id);
+    size_t expect_not_complete = 0U;
+    if (aws_atomic_compare_exchange_int(&token->is_complete, &expect_not_complete, 1U)) {
 
-    if (token->stream_id) {
-        token->closed_fn(token, token->user_data);
+        AWS_LOGF_DEBUG(
+            AWS_LS_EVENT_STREAM_RPC_CLIENT,
+            "token=%p: completing continuation with stream-id %" PRIu32,
+            (void *)token,
+            token->stream_id);
+
+        if (token->stream_id) {
+            token->closed_fn(token, token->user_data);
+        }
+
+        aws_event_stream_rpc_client_continuation_release(token);
     }
-
-    aws_event_stream_rpc_client_continuation_release(token);
 }
 
 static int s_complete_and_clear_each_continuation(void *context, struct aws_hash_element *p_element) {
@@ -353,6 +358,8 @@ void aws_event_stream_rpc_client_connection_release(const struct aws_event_strea
         (void *)connection,
         ref_count - 1);
 
+    AWS_FATAL_ASSERT(ref_count != 0 && "Connection ref count has gone negative");
+
     if (ref_count == 1) {
         s_destroy_connection(connection_mut);
     }
@@ -747,21 +754,31 @@ static void s_route_message_by_type(
         struct aws_hash_element *continuation_element = NULL;
         if (aws_hash_table_find(&connection->continuation_table, &stream_id, &continuation_element) ||
             !continuation_element) {
+            bool old_stream_id = stream_id <= connection->latest_stream_id;
             aws_mutex_unlock(&connection->stream_lock);
-            AWS_LOGF_ERROR(
-                AWS_LS_EVENT_STREAM_RPC_CLIENT,
-                "id=%p: a stream id was received that was not created by this client",
-                (void *)connection);
-            aws_raise_error(AWS_ERROR_EVENT_STREAM_RPC_PROTOCOL_ERROR);
-            s_send_connection_level_error(
-                connection, AWS_EVENT_STREAM_RPC_MESSAGE_TYPE_PROTOCOL_ERROR, 0, &s_invalid_client_stream_id_error);
+            if (!old_stream_id) {
+                AWS_LOGF_ERROR(
+                    AWS_LS_EVENT_STREAM_RPC_CLIENT,
+                    "id=%p: a stream id was received that was not created by this client",
+                    (void *)connection);
+                aws_raise_error(AWS_ERROR_EVENT_STREAM_RPC_PROTOCOL_ERROR);
+                s_send_connection_level_error(
+                    connection, AWS_EVENT_STREAM_RPC_MESSAGE_TYPE_PROTOCOL_ERROR, 0, &s_invalid_client_stream_id_error);
+            } else {
+                AWS_LOGF_WARN(
+                    AWS_LS_EVENT_STREAM_RPC_CLIENT,
+                    "id=%p: a stream id was received that corresponds to an already-closed stream",
+                    (void *)connection);
+            }
             return;
         }
 
+        continuation = continuation_element->value;
+        AWS_FATAL_ASSERT(continuation != NULL);
+        aws_event_stream_rpc_client_continuation_acquire(continuation);
+
         aws_mutex_unlock(&connection->stream_lock);
 
-        continuation = continuation_element->value;
-        aws_event_stream_rpc_client_continuation_acquire(continuation);
         continuation->continuation_fn(continuation, &message_args, continuation->user_data);
         aws_event_stream_rpc_client_continuation_release(continuation);
 
@@ -905,6 +922,7 @@ struct aws_event_stream_rpc_client_continuation_token *aws_event_stream_rpc_clie
     aws_event_stream_rpc_client_connection_acquire(continuation->connection);
     aws_atomic_init_int(&continuation->ref_count, 1);
     aws_atomic_init_int(&continuation->is_closed, 0);
+    aws_atomic_init_int(&continuation->is_complete, 0);
     continuation->continuation_fn = continuation_options->on_continuation;
     continuation->closed_fn = continuation_options->on_continuation_closed;
     continuation->user_data = continuation_options->user_data;
@@ -946,6 +964,8 @@ void aws_event_stream_rpc_client_continuation_release(
         (void *)continuation,
         ref_count - 1);
 
+    AWS_FATAL_ASSERT(ref_count != 0 && "Continuation ref count has gone negative");
+
     if (ref_count == 1) {
         struct aws_allocator *allocator = continuation_mut->connection->allocator;
         aws_event_stream_rpc_client_connection_release(continuation_mut->connection);
@@ -971,7 +991,8 @@ int aws_event_stream_rpc_client_continuation_activate(
     aws_mutex_lock(&continuation->connection->stream_lock);
 
     if (continuation->stream_id) {
-        AWS_LOGF_ERROR(AWS_LS_EVENT_STREAM_RPC_CLIENT, "id=%p: stream has already been activated", (void *)continuation)
+        AWS_LOGF_ERROR(
+            AWS_LS_EVENT_STREAM_RPC_CLIENT, "id=%p: stream has already been activated", (void *)continuation);
         aws_raise_error(AWS_ERROR_INVALID_STATE);
         goto clean_up;
     }
@@ -979,7 +1000,7 @@ int aws_event_stream_rpc_client_continuation_activate(
     /* Even though is_open is atomic, we need to hold a lock while checking it.
      * This lets us coordinate with code that sets is_open to false. */
     if (!aws_event_stream_rpc_client_connection_is_open(continuation->connection)) {
-        AWS_LOGF_ERROR(AWS_LS_EVENT_STREAM_RPC_CLIENT, "id=%p: stream's connection is not open", (void *)continuation)
+        AWS_LOGF_ERROR(AWS_LS_EVENT_STREAM_RPC_CLIENT, "id=%p: stream's connection is not open", (void *)continuation);
         aws_raise_error(AWS_ERROR_EVENT_STREAM_RPC_CONNECTION_CLOSED);
         goto clean_up;
     }
