@@ -8,10 +8,14 @@
 
 #include <aws/http/http.h>
 
+AWS_PUSH_SANE_WARNING_LEVEL
+
 struct aws_client_bootstrap;
 struct aws_socket_options;
+struct aws_socket_endpoint;
 struct aws_tls_connection_options;
 struct aws_http2_setting;
+struct proxy_env_var_settings;
 
 /**
  * An HTTP connection.
@@ -83,13 +87,15 @@ typedef void(aws_http2_on_ping_complete_fn)(
  *      peer, and are safe to retry on another connection.
  * @param http2_error_code The HTTP/2 error code (RFC-7540 section 7) sent by peer.
  *      `enum aws_http2_error_code` lists official codes.
+ * @param debug_data The debug data sent by peer. It can be empty. (NOTE: this data is only valid for the lifetime of
+ *      the callback. Make a deep copy if you wish to keep it longer.)
  * @param user_data User-data passed to the callback.
  */
-
 typedef void(aws_http2_on_goaway_received_fn)(
     struct aws_http_connection *http2_connection,
     uint32_t last_stream_id,
     uint32_t http2_error_code,
+    struct aws_byte_cursor debug_data,
     void *user_data);
 
 /**
@@ -103,6 +109,14 @@ typedef void(aws_http2_on_remote_settings_change_fn)(
     size_t num_settings,
     void *user_data);
 
+/**
+ * Callback invoked on each statistics sample.
+ *
+ * connection_nonce is unique to each connection for disambiguation of each callback per connection.
+ */
+typedef void(
+    aws_http_statistics_observer_fn)(size_t connection_nonce, const struct aws_array_list *stats_list, void *user_data);
+
 /**
  * Configuration options for connection monitoring
  */
@@ -119,59 +133,21 @@ struct aws_http_connection_monitoring_options {
      * as unhealthy.
      */
     uint32_t allowable_throughput_failure_interval_seconds;
-};
-
-/**
- * Supported proxy authentication modes
- */
-enum aws_http_proxy_authentication_type {
-    AWS_HPAT_NONE = 0,
-    AWS_HPAT_BASIC,
-};
-
-/**
- * Options for http proxy server usage
- */
-struct aws_http_proxy_options {
 
     /**
-     * Proxy host to connect to, in lieu of actual target
+     * invoked on each statistics publish by the underlying IO channel. Install this callback to receive the statistics
+     * for observation. This field is optional.
      */
-    struct aws_byte_cursor host;
+    aws_http_statistics_observer_fn *statistics_observer_fn;
 
     /**
-     * Port to make the proxy connection to
+     * user_data to be passed to statistics_observer_fn.
      */
-    uint16_t port;
-
-    /**
-     * Optional.
-     * TLS configuration for the Local <-> Proxy connection
-     * Must be distinct from the the TLS options in the parent aws_http_connection_options struct
-     */
-    const struct aws_tls_connection_options *tls_options;
-
-    /**
-     * What type of proxy authentication to use, if any
-     */
-    enum aws_http_proxy_authentication_type auth_type;
-
-    /**
-     * Optional
-     * User name to use for authentication, basic only
-     */
-    struct aws_byte_cursor auth_username;
-
-    /**
-     * Optional
-     * Password to use for authentication, basic only
-     */
-    struct aws_byte_cursor auth_password;
+    void *statistics_observer_user_data;
 };
 
 /**
  * Options specific to HTTP/1.x connections.
- * Initialize with AWS_HTTP1_CONNECTION_OPTIONS_INIT to set default values.
  */
 struct aws_http1_connection_options {
     /**
@@ -191,7 +167,6 @@ struct aws_http1_connection_options {
 
 /**
  * Options specific to HTTP/2 connections.
- * Initialize with AWS_HTTP2_CONNECTION_OPTIONS_INIT to set default values.
  */
 struct aws_http2_connection_options {
     /**
@@ -204,7 +179,7 @@ struct aws_http2_connection_options {
 
     /**
      * Required
-     * The num of settings to change.
+     * The num of settings to change (Length of the initial_settings_array).
      */
     size_t num_initial_settings;
 
@@ -220,7 +195,7 @@ struct aws_http2_connection_options {
     /**
      * Optional
      * The max number of recently-closed streams to remember.
-     * A default number is set by AWS_HTTP2_CONNECTION_OPTIONS_INIT.
+     * Set it to zero to use the default setting, AWS_HTTP2_DEFAULT_MAX_CLOSED_STREAMS
      *
      * If the connection receives a frame for a closed stream,
      * the frame will be ignored or cause a connection error,
@@ -243,6 +218,26 @@ struct aws_http2_connection_options {
      * See `aws_http2_on_remote_settings_change_fn`.
      */
     aws_http2_on_remote_settings_change_fn *on_remote_settings_change;
+
+    /**
+     * Optional.
+     * Set to true to manually manage the flow-control window of whole HTTP/2 connection.
+     *
+     * If false, the connection will maintain its flow-control windows such that
+     * no back-pressure is applied and data arrives as fast as possible.
+     *
+     * If true, the flow-control window of the whole connection will shrink as body data
+     * is received (headers, padding, and other metadata do not affect the window) for every streams
+     * created on this connection.
+     * The initial connection flow-control window is 65,535.
+     * Once the connection's flow-control window reaches to 0, all the streams on the connection stop receiving any
+     * further data.
+     * The user must call aws_http2_connection_update_window() to increment the connection's
+     * window and keep data flowing.
+     * Note: the padding of data frame counts to the flow-control window.
+     * But, the client will always automatically update the window for padding even for manual window update.
+     */
+    bool conn_manual_window_management;
 };
 
 /**
@@ -264,7 +259,7 @@ struct aws_http_client_connection_options {
 
     /**
      * Required.
-     * Must outlive the connection.
+     * The connection keeps the bootstrap alive via ref-counting.
      */
     struct aws_client_bootstrap *bootstrap;
 
@@ -287,8 +282,8 @@ struct aws_http_client_connection_options {
 
     /**
      * Optional.
-     * aws_http_client_connect() deep-copies all contents except the `aws_tls_ctx`,
-     * which must outlive the the connection.
+     * aws_http_client_connect() deep-copies all contents,
+     * and keeps `aws_tls_ctx` alive via ref-counting.
      */
     const struct aws_tls_connection_options *tls_options;
 
@@ -299,12 +294,28 @@ struct aws_http_client_connection_options {
      */
     const struct aws_http_proxy_options *proxy_options;
 
+    /*
+     * Optional.
+     * Configuration for using proxy from environment variable.
+     * Only works when proxy_options is not set.
+     */
+    const struct proxy_env_var_settings *proxy_ev_settings;
+
     /**
      * Optional
      * Configuration options related to connection health monitoring
      */
     const struct aws_http_connection_monitoring_options *monitoring_options;
 
+    /**
+     * Optional (ignored if 0).
+     * After a request is fully sent, if the server does not begin responding within N milliseconds,
+     * then fail with AWS_ERROR_HTTP_RESPONSE_FIRST_BYTE_TIMEOUT.
+     * This can be overridden per-request by aws_http_make_request_options.response_first_byte_timeout_ms.
+     * TODO: Only supported in HTTP/1.1 now, support it in HTTP/2
+     */
+    uint64_t response_first_byte_timeout_ms;
+
     /**
      * Set to true to manually manage the flow-control window of each stream.
      *
@@ -313,17 +324,26 @@ struct aws_http_client_connection_options {
      *
      * If true, the flow-control window of each stream will shrink as body data
      * is received (headers, padding, and other metadata do not affect the window).
-     * `initial_window_size` determines the starting size of each stream's window.
-     * If a stream's flow-control window reaches 0, no further data will be received.
-     * The user must call aws_http_stream_update_window() to increment the stream's
-     * window and keep data flowing.
+     * `initial_window_size` determines the starting size of each stream's window for HTTP/1 stream, while HTTP/2 stream
+     * will use the settings AWS_HTTP2_SETTINGS_INITIAL_WINDOW_SIZE to inform the other side about read back pressure
+     *
+     * If a stream's flow-control window reaches 0, no further data will be received. The user must call
+     * aws_http_stream_update_window() to increment the stream's window and keep data flowing.
+     *
+     * If a HTTP/2 connection created, it will ONLY control the stream window
+     * management. Connection window management is controlled by
+     * conn_manual_window_management. Note: the padding of data frame counts to the flow-control window.
+     * But, the client will always automatically update the window for padding even for manual window update.
      */
     bool manual_window_management;
 
     /**
-     * The starting size of each HTTP stream's flow-control window.
+     * The starting size of each HTTP stream's flow-control window for HTTP/1 connection.
      * Required if `manual_window_management` is true,
      * ignored if `manual_window_management` is false.
+     *
+     * Always ignored when HTTP/2 connection created. The initial window size is controlled by the settings,
+     * `AWS_HTTP2_SETTINGS_INITIAL_WINDOW_SIZE`
      */
     size_t initial_window_size;
 
@@ -348,6 +368,26 @@ struct aws_http_client_connection_options {
      */
     aws_http_on_client_connection_shutdown_fn *on_shutdown;
 
+    /**
+     * Optional.
+     * When true, use prior knowledge to set up an HTTP/2 connection on a cleartext
+     * connection.
+     * When TLS is set and this is true, the connection will failed to be established,
+     * as prior knowledge only works for cleartext TLS.
+     * Refer to RFC7540 3.4
+     */
+    bool prior_knowledge_http2;
+
+    /**
+     * Optional.
+     * Pointer to the hash map containing the ALPN string to protocol to use.
+     * Hash from `struct aws_string *` to `enum aws_http_version`.
+     * If not set, only the predefined string `h2` and `http/1.1` will be recognized. Other negotiated ALPN string will
+     * result in a HTTP1/1 connection
+     * Note: Connection will keep a deep copy of the table and the strings.
+     */
+    struct aws_hash_table *alpn_string_map;
+
     /**
      * Options specific to HTTP/1.x connections.
      * Optional.
@@ -363,6 +403,19 @@ struct aws_http_client_connection_options {
      * If connection is HTTP/2 and options were not specified, default values are used.
      */
     const struct aws_http2_connection_options *http2_options;
+
+    /**
+     * Optional.
+     * Requests the channel/connection be bound to a specific event loop rather than chosen sequentially from the
+     * event loop group associated with the client bootstrap.
+     */
+    struct aws_event_loop *requested_event_loop;
+
+    /**
+     * Optional
+     * Host resolution override that allows the user to override DNS behavior for this particular connection.
+     */
+    const struct aws_host_resolution_config *host_resolution_config;
 };
 
 /* Predefined settings identifiers (RFC-7540 6.5.2) */
@@ -383,12 +436,6 @@ struct aws_http2_setting {
     uint32_t value;
 };
 
-/**
- * Initializes aws_http1_connection_options with default values.
- */
-#define AWS_HTTP1_CONNECTION_OPTIONS_INIT                                                                              \
-    { .read_buffer_capacity = 0 }
-
 /**
  * HTTP/2: Default value for max closed streams we will keep in memory.
  */
@@ -404,11 +451,6 @@ struct aws_http2_setting {
  */
 #define AWS_HTTP2_SETTINGS_COUNT (6)
 
-/**
- * Initializes aws_http2_connection_options with default values.
- */
-#define AWS_HTTP2_CONNECTION_OPTIONS_INIT                                                                              \
-    { .max_closed_streams = AWS_HTTP2_DEFAULT_MAX_CLOSED_STREAMS }
 /**
  * Initializes aws_http_client_connection_options with default values.
  */
@@ -446,6 +488,13 @@ void aws_http_connection_release(struct aws_http_connection *connection);
 AWS_HTTP_API
 void aws_http_connection_close(struct aws_http_connection *connection);
 
+/**
+ * Stop accepting new requests for the connection. It will NOT start the shutdown process for the connection. The
+ * requests that are already open can still wait to be completed, but new requests will fail to be created,
+ */
+AWS_HTTP_API
+void aws_http_connection_stop_new_requests(struct aws_http_connection *connection);
+
 /**
  * Returns true unless the connection is closed or closing.
  */
@@ -465,13 +514,6 @@ bool aws_http_connection_new_requests_allowed(const struct aws_http_connection *
 AWS_HTTP_API
 bool aws_http_connection_is_client(const struct aws_http_connection *connection);
 
-/**
- * DEPRECATED
- * TODO: Delete once this is removed from H2.
- */
-AWS_HTTP_API
-void aws_http_connection_update_window(struct aws_http_connection *connection, size_t increment_size);
-
 AWS_HTTP_API
 enum aws_http_version aws_http_connection_get_version(const struct aws_http_connection *connection);
 
@@ -482,6 +524,34 @@ enum aws_http_version aws_http_connection_get_version(const struct aws_http_conn
 AWS_HTTP_API
 struct aws_channel *aws_http_connection_get_channel(struct aws_http_connection *connection);
 
+/**
+ * Returns the remote endpoint of the HTTP connection.
+ */
+AWS_HTTP_API
+const struct aws_socket_endpoint *aws_http_connection_get_remote_endpoint(const struct aws_http_connection *connection);
+
+/**
+ * Initialize an map copied from the *src map, which maps `struct aws_string *` to `enum aws_http_version`.
+ */
+AWS_HTTP_API
+int aws_http_alpn_map_init_copy(
+    struct aws_allocator *allocator,
+    struct aws_hash_table *dest,
+    struct aws_hash_table *src);
+
+/**
+ * Initialize an empty hash-table that maps `struct aws_string *` to `enum aws_http_version`.
+ * This map can used in aws_http_client_connections_options.alpn_string_map.
+ */
+AWS_HTTP_API
+int aws_http_alpn_map_init(struct aws_allocator *allocator, struct aws_hash_table *map);
+
+/**
+ * Checks http proxy options for correctness
+ */
+AWS_HTTP_API
+int aws_http_options_validate_proxy_configuration(const struct aws_http_client_connection_options *options);
+
 /**
  * Send a SETTINGS frame (HTTP/2 only).
  * SETTINGS will be applied locally when SETTINGS ACK is received from peer.
@@ -527,7 +597,7 @@ int aws_http2_connection_ping(
  * @param out_settings fixed size array of aws_http2_setting gets set to the local settings
  */
 AWS_HTTP_API
-int aws_http2_connection_get_local_settings(
+void aws_http2_connection_get_local_settings(
     const struct aws_http_connection *http2_connection,
     struct aws_http2_setting out_settings[AWS_HTTP2_SETTINGS_COUNT]);
 
@@ -535,10 +605,10 @@ int aws_http2_connection_get_local_settings(
  * Get the settings received from remote peer, which we are using to restricts the message to send.
  *
  * @param http2_connection HTTP/2 connection.
- * @param out_settings fixed size array of aws_http2_setting gets set to  the remote settings
+ * @param out_settings fixed size array of aws_http2_setting gets set to the remote settings
  */
 AWS_HTTP_API
-int aws_http2_connection_get_remote_settings(
+void aws_http2_connection_get_remote_settings(
     const struct aws_http_connection *http2_connection,
     struct aws_http2_setting out_settings[AWS_HTTP2_SETTINGS_COUNT]);
 
@@ -552,6 +622,8 @@ int aws_http2_connection_get_remote_settings(
  * (http2_error=0, allow_more_streams=true), or to customize the final GOAWAY
  * frame that is sent by this connection.
  *
+ * The other end may not receive the goaway, if the connection already closed.
+ *
  * @param http2_connection HTTP/2 connection.
  * @param http2_error The HTTP/2 error code (RFC-7540 section 7) to send.
  *      `enum aws_http2_error_code` lists official codes.
@@ -563,7 +635,7 @@ int aws_http2_connection_get_remote_settings(
  */
 
 AWS_HTTP_API
-int aws_http2_connection_send_goaway(
+void aws_http2_connection_send_goaway(
     struct aws_http_connection *http2_connection,
     uint32_t http2_error,
     bool allow_more_streams,
@@ -587,7 +659,8 @@ int aws_http2_connection_get_sent_goaway(
 
 /**
  * Get data about the latest GOAWAY frame received from peer (HTTP/2 only).
- * If no GOAWAY has been received, AWS_ERROR_HTTP_DATA_NOT_AVAILABLE will be raised.
+ * If no GOAWAY has been received, or the GOAWAY payload is still in transmitting,
+ * AWS_ERROR_HTTP_DATA_NOT_AVAILABLE will be raised.
  *
  * @param http2_connection HTTP/2 connection.
  * @param out_http2_error Gets set to HTTP/2 error code received in most recent GOAWAY.
@@ -599,6 +672,33 @@ int aws_http2_connection_get_received_goaway(
     uint32_t *out_http2_error,
     uint32_t *out_last_stream_id);
 
+/**
+ * Increment the connection's flow-control window to keep data flowing (HTTP/2 only).
+ *
+ * If the connection was created with `conn_manual_window_management` set true,
+ * the flow-control window of the connection will shrink as body data is received for all the streams created on it.
+ * (headers, padding, and other metadata do not affect the window).
+ * The initial connection flow-control window is 65,535.
+ * Once the connection's flow-control window reaches to 0, all the streams on the connection stop receiving any further
+ * data.
+ *
+ * If `conn_manual_window_management` is false, this call will have no effect.
+ * The connection maintains its flow-control windows such that
+ * no back-pressure is applied and data arrives as fast as possible.
+ *
+ * If you are not connected, this call will have no effect.
+ *
+ * Crashes when the connection is not http2 connection.
+ * The limit of the Maximum Size is 2**31 - 1. If the increment size cause the connection flow window exceeds the
+ * Maximum size, this call will result in the connection lost.
+ *
+ * @param http2_connection HTTP/2 connection.
+ * @param increment_size The size to increment for the connection's flow control window
+ */
+AWS_HTTP_API
+void aws_http2_connection_update_window(struct aws_http_connection *http2_connection, uint32_t increment_size);
+
 AWS_EXTERN_C_END
+AWS_POP_SANE_WARNING_LEVEL
 
 #endif /* AWS_HTTP_CONNECTION_H */
