@@ -441,6 +441,8 @@ int aws_tls_ctx_options_init_default_server(
 }
 
 int aws_tls_ctx_options_set_alpn_list(struct aws_tls_ctx_options *options, const char *alpn_list) {
+    aws_string_destroy(options->alpn_list);
+
     options->alpn_list = aws_string_new_from_c_str(options->allocator, alpn_list);
     if (!options->alpn_list) {
         return AWS_OP_ERR;
@@ -695,7 +697,7 @@ struct aws_channel_handler *aws_tls_server_handler_new(
     struct aws_channel_slot *slot) {
     AWS_FATAL_ASSERT(
         s_client_handler_new &&
-        "For BYO_CRYPTO, you must call aws_tls_server_handler_new_set_callback() with a non-null value.")
+        "For BYO_CRYPTO, you must call aws_tls_server_handler_new_set_callback() with a non-null value.");
     return s_server_handler_new(allocator, options, slot, s_server_user_data);
 }
 
@@ -720,7 +722,7 @@ void aws_tls_byo_crypto_set_server_setup_options(const struct aws_tls_byo_crypto
 int aws_tls_client_handler_start_negotiation(struct aws_channel_handler *handler) {
     AWS_FATAL_ASSERT(
         s_start_negotiation_fn &&
-        "For BYO_CRYPTO, you must call aws_tls_client_handler_set_start_negotiation_callback() with a non-null value.")
+        "For BYO_CRYPTO, you must call aws_tls_client_handler_set_start_negotiation_callback() with a non-null value.");
     return s_start_negotiation_fn(handler, s_client_user_data);
 }
 
