@@ -11,6 +11,8 @@
 
 #include <aws/mqtt/exports.h>
 
+AWS_PUSH_SANE_WARNING_LEVEL
+
 #define AWS_C_MQTT_PACKAGE_ID 5
 
 /* Quality of Service associated with a publish action or subscription [MQTT-4.3]. */
@@ -53,6 +55,31 @@ enum aws_mqtt_error {
     AWS_ERROR_MQTT_CONNECTION_DESTROYED,
     AWS_ERROR_MQTT_CONNECTION_DISCONNECTING,
     AWS_ERROR_MQTT_CANCELLED_FOR_CLEAN_SESSION,
+    AWS_ERROR_MQTT_QUEUE_FULL,
+    AWS_ERROR_MQTT5_CLIENT_OPTIONS_VALIDATION,
+    AWS_ERROR_MQTT5_CONNECT_OPTIONS_VALIDATION,
+    AWS_ERROR_MQTT5_DISCONNECT_OPTIONS_VALIDATION,
+    AWS_ERROR_MQTT5_PUBLISH_OPTIONS_VALIDATION,
+    AWS_ERROR_MQTT5_SUBSCRIBE_OPTIONS_VALIDATION,
+    AWS_ERROR_MQTT5_UNSUBSCRIBE_OPTIONS_VALIDATION,
+    AWS_ERROR_MQTT5_USER_PROPERTY_VALIDATION,
+    AWS_ERROR_MQTT5_PACKET_VALIDATION,
+    AWS_ERROR_MQTT5_ENCODE_FAILURE,
+    AWS_ERROR_MQTT5_DECODE_PROTOCOL_ERROR,
+    AWS_ERROR_MQTT5_CONNACK_CONNECTION_REFUSED,
+    AWS_ERROR_MQTT5_CONNACK_TIMEOUT,
+    AWS_ERROR_MQTT5_PING_RESPONSE_TIMEOUT,
+    AWS_ERROR_MQTT5_USER_REQUESTED_STOP,
+    AWS_ERROR_MQTT5_DISCONNECT_RECEIVED,
+    AWS_ERROR_MQTT5_CLIENT_TERMINATED,
+    AWS_ERROR_MQTT5_OPERATION_FAILED_DUE_TO_OFFLINE_QUEUE_POLICY,
+    AWS_ERROR_MQTT5_ENCODE_SIZE_UNSUPPORTED_PACKET_TYPE,
+    AWS_ERROR_MQTT5_OPERATION_PROCESSING_FAILURE,
+    AWS_ERROR_MQTT5_INVALID_INBOUND_TOPIC_ALIAS,
+    AWS_ERROR_MQTT5_INVALID_OUTBOUND_TOPIC_ALIAS,
+    AWS_ERROR_MQTT5_INVALID_UTF8_STRING,
+    AWS_ERROR_MQTT_CONNECTION_RESET_FOR_ADAPTER_CONNECT,
+    AWS_ERROR_MQTT_CONNECTION_RESUBSCRIBE_NO_TOPICS,
 
     AWS_ERROR_END_MQTT_RANGE = AWS_ERROR_ENUM_END_RANGE(AWS_C_MQTT_PACKAGE_ID),
 };
@@ -61,6 +88,10 @@ enum aws_mqtt_log_subject {
     AWS_LS_MQTT_GENERAL = AWS_LOG_SUBJECT_BEGIN_RANGE(AWS_C_MQTT_PACKAGE_ID),
     AWS_LS_MQTT_CLIENT,
     AWS_LS_MQTT_TOPIC_TREE,
+    AWS_LS_MQTT5_GENERAL,
+    AWS_LS_MQTT5_CLIENT,
+    AWS_LS_MQTT5_CANARY,
+    AWS_LS_MQTT5_TO_MQTT3_ADAPTER,
 };
 
 /** Function called on cleanup of a userdata. */
@@ -70,9 +101,18 @@ AWS_EXTERN_C_BEGIN
 
 AWS_MQTT_API
 bool aws_mqtt_is_valid_topic(const struct aws_byte_cursor *topic);
+
 AWS_MQTT_API
 bool aws_mqtt_is_valid_topic_filter(const struct aws_byte_cursor *topic_filter);
 
+/**
+ * Validate utf-8 string under mqtt specs
+ *
+ * @param text
+ * @return AWS_OP_SUCCESS if the text is validate, otherwise AWS_OP_ERR
+ */
+AWS_MQTT_API int aws_mqtt_validate_utf8_text(struct aws_byte_cursor text);
+
 /**
  * Initializes internal datastructures used by aws-c-mqtt.
  * Must be called before using any functionality in aws-c-mqtt.
@@ -90,5 +130,6 @@ AWS_MQTT_API
 void aws_mqtt_fatal_assert_library_initialized(void);
 
 AWS_EXTERN_C_END
+AWS_POP_SANE_WARNING_LEVEL
 
 #endif /* AWS_MQTT_MQTT_H */
