@@ -162,6 +162,9 @@ struct aws_s3_client_vtable {
     void (*endpoint_shutdown_callback)(struct aws_s3_client *client);
 
     void (*finish_destroy)(struct aws_s3_client *client);
+
+    struct aws_parallel_input_stream *(
+        *parallel_input_stream_new_from_file)(struct aws_allocator *allocator, struct aws_byte_cursor file_name);
 };
 
 /* Represents the state of the S3 client. */
@@ -283,8 +286,8 @@ struct aws_s3_client {
         /* Number of requests sitting in their meta request priority queue, waiting to be streamed. */
         struct aws_atomic_var num_requests_stream_queued_waiting;
 
-        /* Number of requests currently scheduled to be streamed or are actively being streamed. */
-        struct aws_atomic_var num_requests_streaming;
+        /* Number of requests currently scheduled to be streamed the response body or are actively being streamed. */
+        struct aws_atomic_var num_requests_streaming_response;
     } stats;
 
     struct {
@@ -344,7 +347,6 @@ struct aws_s3_client {
 
         /* Number of requests currently being prepared. */
         uint32_t num_requests_being_prepared;
-
     } threaded_data;
 };
 
