@@ -3,6 +3,7 @@
  * SPDX-License-Identifier: Apache-2.0.
  */
 
+#include <aws/s3/private/s3_platform_info.h>
 #include <aws/s3/s3.h>
 
 #include <aws/auth/auth.h>
@@ -68,34 +69,9 @@ static struct aws_log_subject_info_list s_s3_log_subject_list = {
     .count = AWS_ARRAY_SIZE(s_s3_log_subject_infos),
 };
 
-/**** Configuration info for the c5n.18xlarge *****/
-static struct aws_byte_cursor s_c5n_18xlarge_nic_array[] = {AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth0")};
-
-static struct aws_s3_cpu_group_info s_c5n_18xlarge_cpu_group_info_array[] = {
-    {
-        .cpu_group = 0u,
-        .nic_name_array = s_c5n_18xlarge_nic_array,
-        .nic_name_array_length = AWS_ARRAY_SIZE(s_c5n_18xlarge_nic_array),
-    },
-    {
-        .cpu_group = 1u,
-        .nic_name_array = NULL,
-        .nic_name_array_length = 0u,
-    },
-};
-
-static struct aws_s3_compute_platform_info s_c5n_18xlarge_platform_info = {
-    .instance_type = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("c5n.18xlarge"),
-    .max_throughput_gbps = 100u,
-    .cpu_group_info_array = s_c5n_18xlarge_cpu_group_info_array,
-    .cpu_group_info_array_length = AWS_ARRAY_SIZE(s_c5n_18xlarge_cpu_group_info_array),
-};
-/****** End c5n.18xlarge *****/
-
-static struct aws_hash_table s_compute_platform_info_table;
-
 static bool s_library_initialized = false;
 static struct aws_allocator *s_library_allocator = NULL;
+static struct aws_s3_platform_info_loader *s_loader;
 
 void aws_s3_library_init(struct aws_allocator *allocator) {
     if (s_library_initialized) {
@@ -113,66 +89,27 @@ void aws_s3_library_init(struct aws_allocator *allocator) {
 
     aws_register_error_info(&s_error_list);
     aws_register_log_subject_info_list(&s_s3_log_subject_list);
-
-    AWS_FATAL_ASSERT(
-        !aws_hash_table_init(
-            &s_compute_platform_info_table,
-            allocator,
-            32,
-            aws_hash_byte_cursor_ptr_ignore_case,
-            (bool (*)(const void *, const void *))aws_byte_cursor_eq_ignore_case,
-            NULL,
-            NULL) &&
-        "Hash table init failed!");
-
-    AWS_FATAL_ASSERT(
-        !aws_hash_table_put(
-            &s_compute_platform_info_table,
-            &s_c5n_18xlarge_platform_info.instance_type,
-            &s_c5n_18xlarge_platform_info,
-            NULL) &&
-        "hash table put failed!");
-
+    s_loader = aws_s3_platform_info_loader_new(allocator);
+    AWS_FATAL_ASSERT(s_loader);
     s_library_initialized = true;
 }
 
+const struct aws_s3_platform_info *aws_s3_get_current_platform_info(void) {
+    return aws_s3_get_platform_info_for_current_environment(s_loader);
+}
+
 void aws_s3_library_clean_up(void) {
     if (!s_library_initialized) {
         return;
     }
 
     s_library_initialized = false;
+    s_loader = aws_s3_platform_info_loader_release(s_loader);
     aws_thread_join_all_managed();
 
-    aws_hash_table_clean_up(&s_compute_platform_info_table);
     aws_unregister_log_subject_info_list(&s_s3_log_subject_list);
     aws_unregister_error_info(&s_error_list);
     aws_http_library_clean_up();
     aws_auth_library_clean_up();
     s_library_allocator = NULL;
 }
-
-struct aws_s3_compute_platform_info *aws_s3_get_compute_platform_info_for_instance_type(
-    const struct aws_byte_cursor instance_type_name) {
-    AWS_LOGF_TRACE(
-        AWS_LS_S3_GENERAL,
-        "static: looking up compute platform info for instance type " PRInSTR,
-        AWS_BYTE_CURSOR_PRI(instance_type_name));
-
-    struct aws_hash_element *platform_info_element = NULL;
-    aws_hash_table_find(&s_compute_platform_info_table, &instance_type_name, &platform_info_element);
-
-    if (platform_info_element) {
-        AWS_LOGF_INFO(
-            AWS_LS_S3_GENERAL,
-            "static: found compute platform info for instance type " PRInSTR,
-            AWS_BYTE_CURSOR_PRI(instance_type_name));
-        return platform_info_element->value;
-    }
-
-    AWS_LOGF_INFO(
-        AWS_LS_S3_GENERAL,
-        "static: compute platform info for instance type " PRInSTR " not found",
-        AWS_BYTE_CURSOR_PRI(instance_type_name));
-    return NULL;
-}
