@@ -11,7 +11,6 @@
 #include "aws/s3/private/s3_util.h"
 #include <aws/auth/credentials.h>
 #include <aws/common/environment.h>
-#include <aws/common/file.h>
 #include <aws/common/system_info.h>
 #include <aws/common/uri.h>
 #include <aws/http/request_response.h>
@@ -245,7 +244,19 @@ static void s_s3_test_meta_request_progress(
 
     struct aws_s3_meta_request_test_results *meta_request_test_results = user_data;
 
-    aws_atomic_fetch_add(&meta_request_test_results->total_bytes_uploaded, (size_t)progress->bytes_transferred);
+    meta_request_test_results->progress.total_bytes_transferred += progress->bytes_transferred;
+
+    /* Once content_length is reported, it shouldn't change */
+    if (meta_request_test_results->progress.content_length == 0) {
+        meta_request_test_results->progress.content_length = progress->content_length;
+    } else {
+        AWS_FATAL_ASSERT(meta_request_test_results->progress.content_length == progress->content_length);
+    }
+
+    /* If content_length is known, we shouldn't go over it */
+    if (progress->content_length != 0) {
+        AWS_FATAL_ASSERT(meta_request_test_results->progress.total_bytes_transferred <= progress->content_length);
+    }
 
     if (meta_request_test_results->progress_callback != NULL) {
         meta_request_test_results->progress_callback(meta_request, progress, user_data);
@@ -474,7 +485,6 @@ void aws_s3_meta_request_test_results_init(
     AWS_ZERO_STRUCT(*test_meta_request);
     test_meta_request->allocator = allocator;
     aws_atomic_init_int(&test_meta_request->received_body_size_delta, 0);
-    aws_atomic_init_int(&test_meta_request->total_bytes_uploaded, 0);
     aws_array_list_init_dynamic(
         &test_meta_request->synced_data.metrics, allocator, 4, sizeof(struct aws_s3_request_metrics *));
 }
@@ -799,7 +809,7 @@ struct aws_s3_client *aws_s3_tester_mock_client_new(struct aws_s3_tester *tester
     }
 
     aws_atomic_init_int(&mock_client->stats.num_requests_stream_queued_waiting, 0);
-    aws_atomic_init_int(&mock_client->stats.num_requests_streaming, 0);
+    aws_atomic_init_int(&mock_client->stats.num_requests_streaming_response, 0);
 
     return mock_client;
 }
@@ -1324,6 +1334,14 @@ int aws_s3_tester_client_new(
     return AWS_OP_SUCCESS;
 }
 
+/* Disable tsan as we hack into the client threaded data */
+AWS_SUPPRESS_TSAN
+static int s_tester_check_client_thread_data(struct aws_s3_client *client) {
+    ASSERT_UINT_EQUALS(0, client->threaded_data.num_requests_being_prepared);
+    ASSERT_UINT_EQUALS(0, client->threaded_data.request_queue_size);
+    return AWS_OP_SUCCESS;
+}
+
 int aws_s3_tester_send_meta_request_with_options(
     struct aws_s3_tester *tester,
     struct aws_s3_tester_meta_request_options *options,
@@ -1392,6 +1410,7 @@ int aws_s3_tester_send_meta_request_with_options(
 
     struct aws_input_stream *input_stream = NULL;
     struct aws_async_input_stream *async_stream = NULL;
+    size_t upload_size_bytes = 0;
 
     if (meta_request_options.message == NULL) {
         const struct aws_byte_cursor *bucket_name = options->bucket_name;
@@ -1433,17 +1452,17 @@ int aws_s3_tester_send_meta_request_with_options(
              options->default_type_options.mode == AWS_S3_TESTER_DEFAULT_TYPE_MODE_PUT)) {
 
             uint32_t object_size_mb = options->put_options.object_size_mb;
-            size_t object_size_bytes = (size_t)object_size_mb * 1024ULL * 1024ULL;
+            upload_size_bytes = (size_t)object_size_mb * 1024ULL * 1024ULL;
 
             /* This doesn't do what we think it should because
              * g_min_upload_part_size overrides client->part_size */
             if (options->put_options.ensure_multipart) {
-                if (object_size_bytes == 0) {
-                    object_size_bytes = client->part_size * 2;
-                    object_size_mb = (uint32_t)(object_size_bytes / 1024 / 1024);
+                if (upload_size_bytes == 0) {
+                    upload_size_bytes = client->part_size * 2;
+                    object_size_mb = (uint32_t)(upload_size_bytes / 1024 / 1024);
                 }
 
-                ASSERT_TRUE(object_size_bytes > client->part_size);
+                ASSERT_TRUE(upload_size_bytes > client->part_size);
             }
 
             struct aws_byte_buf object_path_buffer;
@@ -1503,7 +1522,7 @@ int aws_s3_tester_send_meta_request_with_options(
             struct aws_async_input_stream_tester_options stream_options = {
                 .base =
                     {
-                        .autogen_length = object_size_bytes,
+                        .autogen_length = upload_size_bytes,
                         .eof_requires_extra_read = options->put_options.eof_requires_extra_read,
                         .max_bytes_per_read = options->put_options.max_bytes_per_read,
                     },
@@ -1528,35 +1547,7 @@ int aws_s3_tester_send_meta_request_with_options(
             /* if uploading via filepath, write input_stream out as tmp file on disk, and then upload that */
             if (options->put_options.file_on_disk) {
                 ASSERT_NOT_NULL(input_stream);
-                struct aws_byte_buf filepath_buf;
-                aws_byte_buf_init(&filepath_buf, allocator, 128);
-                struct aws_byte_cursor filepath_prefix = aws_byte_cursor_from_c_str("tmp");
-                aws_byte_buf_append_dynamic(&filepath_buf, &filepath_prefix);
-                aws_byte_buf_append_dynamic(&filepath_buf, &test_object_path);
-                for (size_t i = 0; i < filepath_buf.len; ++i) {
-                    if (!isalnum(filepath_buf.buffer[i])) {
-                        filepath_buf.buffer[i] = '_'; /* sanitize filename */
-                    }
-                }
-                filepath_str = aws_string_new_from_buf(allocator, &filepath_buf);
-                aws_byte_buf_clean_up(&filepath_buf);
-
-                FILE *file = aws_fopen(aws_string_c_str(filepath_str), "wb");
-                ASSERT_NOT_NULL(file, "Cannot open file for write: %s", aws_string_c_str(filepath_str));
-
-                int64_t stream_length = 0;
-                ASSERT_SUCCESS(aws_input_stream_get_length(input_stream, &stream_length));
-
-                struct aws_byte_buf data_buf;
-                ASSERT_SUCCESS(aws_byte_buf_init(&data_buf, allocator, (size_t)stream_length));
-                ASSERT_SUCCESS(aws_input_stream_read(input_stream, &data_buf));
-                ASSERT_UINT_EQUALS((size_t)stream_length, data_buf.len);
-
-                ASSERT_UINT_EQUALS(data_buf.len, fwrite(data_buf.buffer, 1, data_buf.len, file));
-                fclose(file);
-                aws_byte_buf_clean_up(&data_buf);
-
-                /* use filepath instead of input_stream */
+                filepath_str = aws_s3_tester_create_file(allocator, test_object_path, input_stream);
                 meta_request_options.send_filepath = aws_byte_cursor_from_string(filepath_str);
                 input_stream = aws_input_stream_release(input_stream);
             }
@@ -1572,7 +1563,7 @@ int aws_s3_tester_send_meta_request_with_options(
                     &host_cur,
                     g_test_body_content_type,
                     test_object_path,
-                    object_size_bytes,
+                    upload_size_bytes,
                     options->sse_type);
             }
 
@@ -1614,6 +1605,14 @@ int aws_s3_tester_send_meta_request_with_options(
         aws_string_destroy(host_name);
     } else {
         aws_http_message_acquire(meta_request_options.message);
+
+        if (options->meta_request_type == AWS_S3_META_REQUEST_TYPE_PUT_OBJECT) {
+            /* Figure out how much is being uploaded from pre-existing message */
+            struct aws_input_stream *mystery_stream = aws_http_message_get_body_stream(meta_request_options.message);
+            if (mystery_stream != NULL) {
+                ASSERT_SUCCESS(aws_input_stream_get_length(mystery_stream, (int64_t *)&upload_size_bytes));
+            }
+        }
     }
 
     struct aws_s3_meta_request_test_results meta_request_test_results;
@@ -1648,21 +1647,6 @@ int aws_s3_tester_send_meta_request_with_options(
         ASSERT_TRUE(aws_s3_meta_request_is_finished(meta_request));
     }
 
-    /* If total_bytes_uploaded wasn't set by the progress callback,
-     * then set it based on number of bytes read from input-stream.
-     * (progress callback isn't currently hooked up for single part upload) */
-    if ((out_results->finished_error_code == AWS_ERROR_SUCCESS) &&
-        (aws_atomic_load_int(&out_results->total_bytes_uploaded) == 0)) {
-
-        uint64_t bytes_read_from_stream = 0;
-        if (input_stream != NULL) {
-            bytes_read_from_stream = aws_input_stream_tester_total_bytes_read(input_stream);
-        } else if (async_stream != NULL) {
-            bytes_read_from_stream = aws_async_input_stream_tester_total_bytes_read(async_stream);
-        }
-        aws_atomic_store_int(&out_results->total_bytes_uploaded, (size_t)bytes_read_from_stream);
-    }
-
     switch (options->validate_type) {
         case AWS_S3_TESTER_VALIDATE_TYPE_EXPECT_SUCCESS:
             ASSERT_INT_EQUALS(AWS_ERROR_SUCCESS, out_results->finished_error_code);
@@ -1671,7 +1655,23 @@ int aws_s3_tester_send_meta_request_with_options(
                 ASSERT_SUCCESS(aws_s3_tester_validate_get_object_results(out_results, options->sse_type));
             } else if (meta_request_options.type == AWS_S3_META_REQUEST_TYPE_PUT_OBJECT) {
                 ASSERT_SUCCESS(aws_s3_tester_validate_put_object_results(out_results, options->sse_type));
+
+                /* Expected number of bytes should have been read from stream, and reported via progress callbacks */
+                if (input_stream != NULL) {
+                    ASSERT_UINT_EQUALS(upload_size_bytes, aws_input_stream_tester_total_bytes_read(input_stream));
+                } else if (async_stream != NULL) {
+                    ASSERT_UINT_EQUALS(upload_size_bytes, aws_async_input_stream_tester_total_bytes_read(async_stream));
+                }
+
+                ASSERT_UINT_EQUALS(upload_size_bytes, out_results->progress.total_bytes_transferred);
+                if (!options->put_options.skip_content_length) {
+                    ASSERT_UINT_EQUALS(upload_size_bytes, out_results->progress.content_length);
+                }
             }
+            ASSERT_UINT_EQUALS(0, aws_atomic_load_int(&client->stats.num_requests_in_flight));
+            ASSERT_UINT_EQUALS(0, aws_atomic_load_int(&client->stats.num_requests_stream_queued_waiting));
+            ASSERT_UINT_EQUALS(0, aws_atomic_load_int(&client->stats.num_requests_streaming_response));
+            ASSERT_SUCCESS(s_tester_check_client_thread_data(client));
             break;
         case AWS_S3_TESTER_VALIDATE_TYPE_EXPECT_FAILURE:
             ASSERT_FALSE(out_results->finished_error_code == AWS_ERROR_SUCCESS);
@@ -1857,6 +1857,8 @@ int aws_s3_tester_validate_get_object_results(
         meta_request_test_results->received_body_size);
 
     ASSERT_TRUE(content_length == meta_request_test_results->received_body_size);
+    ASSERT_UINT_EQUALS(content_length, meta_request_test_results->progress.total_bytes_transferred);
+    ASSERT_UINT_EQUALS(content_length, meta_request_test_results->progress.content_length);
 
     return AWS_OP_SUCCESS;
 }
@@ -2016,3 +2018,38 @@ int aws_s3_tester_get_content_length(const struct aws_http_headers *headers, uin
     ASSERT_SUCCESS(aws_byte_cursor_utf8_parse_u64(value_cursor, out_content_length));
     return AWS_OP_SUCCESS;
 }
+
+struct aws_string *aws_s3_tester_create_file(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor test_object_path,
+    struct aws_input_stream *input_stream) {
+
+    struct aws_byte_buf filepath_buf;
+    aws_byte_buf_init(&filepath_buf, allocator, 128);
+    struct aws_byte_cursor filepath_prefix = aws_byte_cursor_from_c_str("tmp");
+    aws_byte_buf_append_dynamic(&filepath_buf, &filepath_prefix);
+    aws_byte_buf_append_dynamic(&filepath_buf, &test_object_path);
+    for (size_t i = 0; i < filepath_buf.len; ++i) {
+        if (!isalnum(filepath_buf.buffer[i])) {
+            filepath_buf.buffer[i] = '_'; /* sanitize filename */
+        }
+    }
+    struct aws_string *filepath_str = aws_string_new_from_buf(allocator, &filepath_buf);
+    aws_byte_buf_clean_up(&filepath_buf);
+
+    FILE *file = aws_fopen(aws_string_c_str(filepath_str), "wb");
+    AWS_FATAL_ASSERT(file != NULL);
+
+    int64_t stream_length = 0;
+    AWS_FATAL_ASSERT(aws_input_stream_get_length(input_stream, &stream_length) == AWS_OP_SUCCESS);
+
+    struct aws_byte_buf data_buf;
+    AWS_FATAL_ASSERT(aws_byte_buf_init(&data_buf, allocator, (size_t)stream_length) == AWS_OP_SUCCESS);
+    AWS_FATAL_ASSERT(aws_input_stream_read(input_stream, &data_buf) == AWS_OP_SUCCESS);
+    AWS_FATAL_ASSERT((size_t)stream_length == data_buf.len);
+    AWS_FATAL_ASSERT(data_buf.len == fwrite(data_buf.buffer, 1, data_buf.len, file));
+    fclose(file);
+    aws_byte_buf_clean_up(&data_buf);
+
+    return filepath_str;
+}
