@@ -15,6 +15,7 @@
 
 #include <aws/common/common.h>
 #include <aws/common/condition_variable.h>
+#include <aws/common/file.h>
 #include <aws/common/logging.h>
 #include <aws/common/mutex.h>
 #include <aws/common/string.h>
@@ -233,10 +234,11 @@ struct aws_s3_meta_request_test_results {
     int finished_error_code;
     enum aws_s3_checksum_algorithm algorithm;
 
-    /* accumulator of amount of bytes uploaded.
-     * Currently, this only works for MPU and Copy meta-requests.
-     * It's powered by the progress_callback which isn't invoked for all types */
-    struct aws_atomic_var total_bytes_uploaded;
+    /* Record data from progress_callback() */
+    struct {
+        uint64_t content_length;          /* Remember progress->content_length */
+        uint64_t total_bytes_transferred; /* Accumulator for progress->bytes_transferred */
+    } progress;
 
     /* Protected the tester->synced_data.lock */
     struct {
@@ -445,8 +447,18 @@ int aws_s3_tester_upload_file_path_init(
     struct aws_byte_buf *out_path_buffer,
     struct aws_byte_cursor file_path);
 
+/* Create a file on disk based on the input stream. Return the file path */
+struct aws_string *aws_s3_tester_create_file(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor test_object_path,
+    struct aws_input_stream *input_stream);
+
 int aws_s3_tester_get_content_length(const struct aws_http_headers *headers, uint64_t *out_content_length);
 
+struct aws_parallel_input_stream *aws_parallel_input_stream_new_from_file_failure_tester(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor file_name);
+
 extern struct aws_s3_client_vtable g_aws_s3_client_mock_vtable;
 
 extern const struct aws_byte_cursor g_mock_server_uri;
