@@ -525,9 +525,8 @@ int EVP_PKEY_paramgen(EVP_PKEY_CTX *ctx, EVP_PKEY **out_pkey) {
 int EVP_PKEY_encapsulate(EVP_PKEY_CTX *ctx,
                          uint8_t *ciphertext, size_t *ciphertext_len,
                          uint8_t *shared_secret, size_t *shared_secret_len) {
-  if (ctx == NULL ||
-      ctx->pmeth == NULL ||
-      ctx->pmeth->encapsulate == NULL) {
+  if (ctx == NULL || ctx->pmeth == NULL || ctx->pmeth->encapsulate == NULL) {
+      OPENSSL_PUT_ERROR(EVP, EVP_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE);
       return 0;
   }
 
@@ -538,9 +537,8 @@ int EVP_PKEY_encapsulate(EVP_PKEY_CTX *ctx,
 int EVP_PKEY_decapsulate(EVP_PKEY_CTX *ctx,
                          uint8_t *shared_secret, size_t *shared_secret_len,
                          uint8_t *ciphertext, size_t ciphertext_len) {
-  if (ctx == NULL ||
-      ctx->pmeth == NULL ||
-      ctx->pmeth->decapsulate == NULL) {
+  if (ctx == NULL || ctx->pmeth == NULL || ctx->pmeth->decapsulate == NULL) {
+      OPENSSL_PUT_ERROR(EVP, EVP_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE);
       return 0;
   }
 
