@@ -799,6 +799,12 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(s3_client_new, METH_VARARGS),
     AWS_PY_METHOD_DEF(s3_client_make_meta_request, METH_VARARGS),
     AWS_PY_METHOD_DEF(s3_meta_request_cancel, METH_VARARGS),
+    AWS_PY_METHOD_DEF(s3_get_ec2_instance_type, METH_NOARGS),
+    AWS_PY_METHOD_DEF(s3_is_crt_s3_optimized_for_system, METH_NOARGS),
+    AWS_PY_METHOD_DEF(s3_get_recommended_throughput_target_gbps, METH_NOARGS),
+    AWS_PY_METHOD_DEF(s3_cross_process_lock_new, METH_VARARGS),
+    AWS_PY_METHOD_DEF(s3_cross_process_lock_acquire, METH_VARARGS),
+    AWS_PY_METHOD_DEF(s3_cross_process_lock_release, METH_VARARGS),
 
     /* WebSocket */
     AWS_PY_METHOD_DEF(websocket_client_connect, METH_VARARGS),
